/*
 *  Armadillo Workflow Platform v1.0
 *  A simple pipeline system for phylogenetic analysis
 *  
 *  Copyright (C) 2009-2011  Etienne Lord, Mickael Leclercq
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package Scripts;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Vector;
import workflows.armadillo_workflow;
import workflows.WorkflowModel;
//mport workflows.WorkflowModel.workflow_object;
import workflows.workflow_properties;

/**
 * This object manage transformation from workflow to script format.
 *
 * @author Alix Boc, Etienne Lord
 * @since June 2009
 */

public class Scripts_conversion {

    HashMap<String, workflow_properties>Visited=new HashMap<String, workflow_properties>();

    ////////////////////////////////////////////////////////////////////////////
    /// Armadillo Workflow
    
    public void TriTopo(armadillo_workflow.Workflow G, LinkedList<armadillo_workflow.workflow_object> Sorted) {
          //--Reset find flag
         Visited.clear();
         for (armadillo_workflow.workflow_object obj:G.work) obj.getProperties().remove("Order");
         //--Tri Topo
         for (armadillo_workflow.workflow_object obj:G.work) {
             Visit(G, obj, Sorted);
          }
          //--Inverse list and add Order...
         Collections.reverse(Sorted);
         for (int i=0; i<Sorted.size();i++) {
             armadillo_workflow.workflow_object o=Sorted.get(i);
             o.getProperties().put("Order", i);
         }
          
    }

      private void Visit(armadillo_workflow.Workflow G, armadillo_workflow.workflow_object Sommet,LinkedList<armadillo_workflow.workflow_object> Sorted) {
         if (!Visited.containsKey(Sommet.getProperties().getID())) {
             Visited.put(Sommet.getProperties().getID(), Sommet.getProperties());
             for (armadillo_workflow.workflow_object obj:ChercherNextVoisin(G,Sommet)) {
                 Visit(G,obj,Sorted);                
             }
              Sorted.add(Sommet);
         }
      }

    private Vector<armadillo_workflow.workflow_object> ChercherNextVoisin(armadillo_workflow.Workflow G, armadillo_workflow.workflow_object Sommet) {
        Vector<armadillo_workflow.workflow_object>tmp=new Vector<armadillo_workflow.workflow_object>();      
        for (armadillo_workflow.workflow_object obj:G.findOutput(Sommet)) tmp.add(obj);
        return tmp;
    }

    ////////////////////////////////////////////////////////////////////////////
    /// workflow (new version)
    
    public void TriTopo(WorkflowModel.Workflow G, LinkedList<WorkflowModel.workflow_object> Sorted) {
          //--Reset find flag
         Visited.clear();
         for (WorkflowModel.workflow_object obj:G.work) obj.getProperties().remove("Order");
         //--Tri Topo
         for (WorkflowModel.workflow_object obj:G.work) {
             Visit(G, obj, Sorted);
          }
          //--Inverse list and add Order...
         Collections.reverse(Sorted);
         for (int i=0; i<Sorted.size();i++) {
             WorkflowModel.workflow_object o=Sorted.get(i);
             o.getProperties().put("Order", i);
         }
          
    }

      private void Visit(WorkflowModel.Workflow G, WorkflowModel.workflow_object Sommet,LinkedList<WorkflowModel.workflow_object> Sorted) {
         if (!Visited.containsKey(Sommet.getProperties().getID())) {
             Visited.put(Sommet.getProperties().getID(), Sommet.getProperties());
             for (WorkflowModel.workflow_object obj:ChercherNextVoisin(G,Sommet)) {
                 Visit(G,obj,Sorted);                
             }
              Sorted.add(Sommet);
         }
      }

    private Vector<WorkflowModel.workflow_object> ChercherNextVoisin(WorkflowModel.Workflow G, WorkflowModel.workflow_object Sommet) {
        Vector<WorkflowModel.workflow_object>tmp=new Vector<WorkflowModel.workflow_object>();      
        for (WorkflowModel.workflow_object obj:G.findOutput(Sommet)) tmp.add(obj);
        return tmp;
    }
  
}
