/*
 *  Armadillo Workflow Platform v1.0
 *  A simple pipeline system for phylogenetic analysis
 *  
 *  Copyright (C) 2009-2011  Etienne Lord, Mickael Leclercq
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package biologic;

import configuration.Util;
import database.databaseFunction;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import workflows.workflow_properties;

/**
 * This is a simple Matrix
 *
 * @author Etienne Lord
 * @since 2010
 */
public class Matrix implements Biologic, Iterator, Serializable {
 
    private int id=0;
    private String matrix="";
    private String name="UnkwnownMatrix";
    private String note="";
    private int runProgram_id=0;
    public static databaseFunction df=new databaseFunction();

     public Matrix() {}

     public Matrix(int id) {
          this.loadFromDatabase(id);
     }

     /**
      * This
      * @param filename
      */
     public Matrix(String filename) {
         try {
         String[] stri=Util.InputFile(filename);
         for (String s:stri) {
             matrix+=s+"\n";
         }       
         } catch(Exception e) {e.printStackTrace();matrix="";}
     }

      public boolean loadFromFile(String filename) {
        try {
         String[] stri=Util.InputFile(filename);
         for (String s:stri) {
             matrix+=s+"\n";
         }
         } catch(Exception e) {e.printStackTrace();matrix="";}
        return true;
    }

      ////////////////////////////////////////////////////////////////////////////////
    // Database function

    public boolean loadFromDatabase(int id) {
        Matrix un=df.getMatrix(id);
        if (un.id>0) {
            this.setMatrix(un.getMatrix());
            this.setName(un.getName());
            this.setNote(un.getNote());
            this.setRunProgram_id(un.getRunProgram_id());
            this.id=id;
            return true;
        } else {
            return false;
        }
    }

    public boolean saveToDatabase() {
        id=0;
        id=df.addMatrix(this);
        return (id==0?false:true);
    }

    public boolean removeFromDatabase() {
        return df.removeMatrix(this);
    }

    public boolean updateDatabase() {
        return df.updateMatrix(this);
    }

////////////////////////////////////////////////////////////////////////////////
/// Iterator

     Vector<Integer>next=new Vector<Integer>();
    int counter=0;
    int maxid=-1;
    public boolean hasNext() {
       //next=df.getAllSequenceID();
       if (next.size()==0) {
           next=df.getAllMatrixID();
           maxid=next.size();
       }
       return (this.counter<maxid);
    }

    public Object next() {
        return new Matrix(next.get(counter++));
    }

    public void remove() {
        Matrix s=new Matrix(counter-1);
        s.removeFromDatabase();
    }

     @Override
    public Vector<Integer> getAllId() {
        return next;
    }
    
    public boolean exists(Integer id) {
        return df.existsMatrix(id);
    }

    ////////////////////////////////////////////////////////////////////////////

    /**
     * This save the matrix data to a file
     * @param filename
     * @return True if successfull
     */
    public boolean Output(String filename) {
        try {
            PrintWriter pw=new PrintWriter(new FileWriter(new File(filename)));
            pw.print(matrix);
            pw.flush();
            pw.close();
            return true;
        } catch(Exception e) {return false;}
    }



    /**
     * @return the id
     */
    public int getId() {
        return id;
    }

    /**
     * @param id the id to set
     */
    public void setId(int id) {
        this.id = id;
    }

    /**
     * @return the matrix
     */
    public String getMatrix() {
        return matrix;
    }

    /**
     * @param matrix the matrix to set
     */
    public void setMatrix(String matrix) {
        this.matrix = matrix;
    }

    /**
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return the note
     */
    public String getNote() {
        return note;
    }

    /**
     * @param note the note to set
     */
    public void setNote(String note) {
        this.note = note;
    }

    /**
     * @return the runProgram_id
     */
    public int getRunProgram_id() {
        return runProgram_id;
    }

    /**
     * @param runProgram_id the runProgram_id to set
     */
    public void setRunProgram_id(int runProgram_id) {
        this.runProgram_id = runProgram_id;
    }
  
    public String getBiologicType() {
        return "Matrix";
    }

    public workflow_properties returnProperties() {
         workflow_properties tmp=new workflow_properties();
         if (id==0) this.saveToDatabase();
             tmp.put("input_matrix_id", this.getId());
             tmp.put("output_matrix_id", this.getId());
         return tmp;
     }

    @Override
    public String toString() {
        return this.getMatrix()+"\n"+this.getNote();
    }


//-- Not implemented
//    Vector<Vector<Object>> matrix=new Vector<Vector<Object>>();
//    Vector<String>         column_name=new Vector<String>();
//    Vector<String>         row_name=new Vector<String>();
//
//    Vector current_line;
//
//    public void AddLine() {
//         Vector<Object> line;
//        if (current_line!=null) {
//             line=new Vector<Object>(current_line.size());
//        } else  {
//             line=new Vector<Object>();
//        }
//        matrix.add(line);
//        current_line=line;
//    };
//
//    /**
//     * This add a Colum to the present line
//     * The
//     * @param obj
//     */
//    public void AddColumn(Object obj) {
//        if (current_line!=null) {
//            current_line.add(obj);
//        } else {
//           AddLine();
//           AddColumn(obj);
//        }
//    };

    public String toHtml() {
        String str=this.matrix.replaceAll("\n", "<br>");
        return str;
    }

    public String getNameId(int id) {
        return df.getMatrixName(id);
    }

    public void setData(String data) {
        this.setMatrix(data);
    }

    public String getFasta() {
       return "";
    }

    public String getPhylip() {
       return "";
    }

    public String getExtendedString() {
        return toString();
    }
    
    /**
     * Get the next valid Id found in the database
     * @param start the current workflow id...
     * @return the next workflow id or 0 if not found
     */
    public int getNextValidId() {
        if (!hasNext()) return 0;
        for (int i=0;i<next.size();i++) {
            if (next.get(i)==this.id) return next.get(i+1);
        }
        //--Unable to find the current ID? -> Find the first higher...
         for (int i=0;i<next.size();i++) {
            if (next.get(i)>this.id) return next.get(i);
        }
        //--If not, return 0; 
        return 0;
    }
    
    /**
     * This replace any reference found in the text with the correct
     * Sequence name
     */
    public void ReplaceResultsWithSequenceName() {        
      //--Replace name with correct sequence name...
      String text=getMatrix();
      Pattern find_sequence=Pattern.compile("AZ([0-9]*)");
      Matcher m=find_sequence.matcher(text);
      while(m.find()) {
              try {
                  Sequence sequence=new Sequence(Integer.valueOf(m.group(1)));
                  if (sequence!=null) {
                    String group=m.group();
                    text=text.replace(group, sequence.getName());                    
                  }
              } catch(Exception e) {}              
      }
      setMatrix(text);
    }
}
